@@ -17,32 +17,57 @@ from abc import ABC, abstractmethod
 from contextlib import contextmanager
 from enum import Enum, auto
 from pathlib import Path
-from time import time
 from types import TracebackType
-from typing import Optional, Type, Generator, Any
+from typing import Optional, Type, Generator, Any, Iterable, Union
+from ledgered.devices import Device
+from warnings import warn
 
-from ragger.firmware import Firmware
-from ragger.utils import pack_APDU, RAPDU, Crop
+from ragger.firmware import DEPRECATION_MESSAGE, Firmware
 from ragger.logger import get_default_logger, get_apdu_logger, set_apdu_logger_file
+from ragger.utils import pack_APDU, RAPDU, Crop
 
 
 class RaisePolicy(Enum):
     RAISE_NOTHING = auto()
     RAISE_ALL_BUT_0x9000 = auto()
     RAISE_ALL = auto()
+    RAISE_CUSTOM = auto()
+
+
+class GraphicalLibrary(Enum):
+    BAGL = auto()
+    NBGL = auto()
+
+    @classmethod
+    def from_string(cls, graphic_str: str) -> "GraphicalLibrary":
+        lookup = {
+            "bagl": cls.BAGL,
+            "nbgl": cls.NBGL,
+        }
+        if graphic_str in lookup:
+            return lookup[graphic_str.lower()]
+        raise ValueError(f"Unknown graphical library: {graphic_str}")
+
+    def __str__(self) -> str:
+        return self.name.lower()
 
 
 class BackendInterface(ABC):
 
-    def __init__(self, firmware: Firmware, log_apdu_file: Optional[Path] = None):
+    def __init__(self,
+                 device: Device,
+                 log_apdu_file: Optional[Path] = None,
+                 whitelisted_status: Iterable = ()):
         """Initializes the Backend
 
-        :param firmware: Which Firmware will be managed
-        :type firmware: Firmware
+        :param device: Which Device will be managed
+        :type device: Device
         """
-        self._firmware = firmware
+        self._device = device
         self._last_async_response: Optional[RAPDU] = None
         self.raise_policy = RaisePolicy.RAISE_ALL_BUT_0x9000
+        # SDK graphic library
+        self.sdk_graphics: Optional[GraphicalLibrary] = None
 
         if log_apdu_file:
             set_apdu_logger_file(log_apdu_file=log_apdu_file)
@@ -50,13 +75,24 @@ class BackendInterface(ABC):
         self.logger = get_default_logger()
         self.apdu_logger = get_apdu_logger()
 
+        self.whitelisted_status = whitelisted_status
+
+    @property
+    def device(self) -> Device:
+        """
+        :return: The currently managed Device.
+        :rtype: Device
+        """
+        return self._device
+
     @property
     def firmware(self) -> Firmware:
         """
-        :return: The currently managed Firmware.
-        :rtype: Firmware
+        :return: The currently managed Device.
+        :rtype: Device
         """
-        return self._firmware
+        warn(DEPRECATION_MESSAGE)
+        return Firmware(self._device.type)
 
     @property
     def last_async_response(self) -> Optional[RAPDU]:
@@ -94,7 +130,9 @@ class BackendInterface(ABC):
         """
         return ((self.raise_policy == RaisePolicy.RAISE_ALL)
                 or ((self.raise_policy == RaisePolicy.RAISE_ALL_BUT_0x9000) and
-                    (rapdu.status != 0x9000)))
+                    (rapdu.status != 0x9000))
+                or ((self.raise_policy == RaisePolicy.RAISE_CUSTOM) and
+                    (rapdu.status not in self.whitelisted_status)))
 
     def send(self, cla: int, ins: int, p1: int = 0, p2: int = 0, data: bytes = b"") -> None:
         """
@@ -106,8 +144,8 @@ class BackendInterface(ABC):
         :type ins: int
         :param p1: First instruction parameter, defaults to 0
         :type p1: int
-        :param p1: Second instruction parameter, defaults to 0
-        :type p1: int
+        :param p2: Second instruction parameter, defaults to 0
+        :type p2: int
         :param data: Command data
         :type data: bytes
 
@@ -149,7 +187,13 @@ class BackendInterface(ABC):
         """
         raise NotImplementedError
 
-    def exchange(self, cla: int, ins: int, p1: int = 0, p2: int = 0, data: bytes = b"") -> RAPDU:
+    def exchange(self,
+                 cla: int,
+                 ins: int,
+                 p1: int = 0,
+                 p2: int = 0,
+                 data: bytes = b"",
+                 tick_timeout: int = 5 * 60 * 10) -> RAPDU:
         """
         Formats and sends an APDU to the backend, then receives its response.
 
@@ -161,8 +205,8 @@ class BackendInterface(ABC):
         :type ins: int
         :param p1: First instruction parameter, defaults to 0
         :type p1: int
-        :param p1: Second instruction parameter, defaults to 0
-        :type p1: int
+        :param p2: Second instruction parameter, defaults to 0
+        :type p2: int
         :param data: Command data
         :type data: bytes
 
@@ -173,10 +217,10 @@ class BackendInterface(ABC):
         :return: The APDU response
         :rtype: RAPDU
         """
-        return self.exchange_raw(pack_APDU(cla, ins, p1, p2, data))
+        return self.exchange_raw(pack_APDU(cla, ins, p1, p2, data), tick_timeout=tick_timeout)
 
     @abstractmethod
-    def exchange_raw(self, data: bytes = b"") -> RAPDU:
+    def exchange_raw(self, data: bytes = b"", tick_timeout: int = 5 * 60 * 10) -> RAPDU:
         """
         Sends the given APDU to the backend, then receives its response.
 
@@ -218,8 +262,8 @@ class BackendInterface(ABC):
         :type ins: int
         :param p1: First instruction parameter, defaults to 0
         :type p1: int
-        :param p1: Second instruction parameter, defaults to 0
-        :type p1: int
+        :param p2: Second instruction parameter, defaults to 0
+        :type p2: int
         :param data: Command data
         :type data: bytes
 
@@ -235,7 +279,7 @@ class BackendInterface(ABC):
 
     @contextmanager
     @abstractmethod
-    def exchange_async_raw(self, data: bytes = b"") -> Generator[None, None, None]:
+    def exchange_async_raw(self, data: bytes = b"") -> Generator[Union[bool, None], None, None]:
         """
         Sends the given APDU to the backend, then gives the control back to the
         caller.
@@ -248,7 +292,7 @@ class BackendInterface(ABC):
 
         :raises ExceptionRAPDU: If the `raises` attribute is True, this method
                                   will raise if the backend returns a status code
-                                  not registered a a `valid_statuses`
+                                  not registered as a `valid_statuses`
 
         :return: None
         :rtype: NoneType
@@ -330,6 +374,37 @@ class BackendInterface(ABC):
         """
         raise NotImplementedError
 
+    @abstractmethod
+    def finger_swipe(self,
+                     x: int = 0,
+                     y: int = 0,
+                     direction: str = "left",
+                     delay: float = 0.5) -> None:
+        """
+        Performs a finger swipe on the device screen.
+
+        This method may be left void on backends connecting to physical devices,
+        where a physical interaction must be performed instead.
+        This will prevent the instrumentation to fail (the void method won't
+        raise `NotImplementedError`), but the instrumentation flow will probably
+        get stuck (on further call to `receive` for instance) until the expected
+        action is performed on the device.
+
+        :param x: The x coordinate of the initial finger touch.
+        :type x: int
+        :param y: The y coordinate of the initial finger touch.
+        :type y: int
+        :param direction: The direction where to orientate the swipe. Must be in: ['up', 'down',
+                          'left', 'right]
+        :type direction: str
+        :param delay: Delay between finger touch press and release actions.
+        :type delay: float
+
+        :return: None
+        :rtype: NoneType
+        """
+        raise NotImplementedError
+
     @abstractmethod
     def compare_screen_with_snapshot(self,
                                      golden_snap_path: Path,
@@ -378,6 +453,7 @@ class BackendInterface(ABC):
         """
         raise NotImplementedError
 
+    @abstractmethod
     def wait_for_home_screen(self, timeout: float = 10.0) -> None:
         """
         Wait until the screen content is equal to the app home screen.
@@ -419,6 +495,7 @@ class BackendInterface(ABC):
         """
         raise NotImplementedError
 
+    @abstractmethod
     def wait_for_text_on_screen(self, text: str, timeout: float = 10.0) -> None:
         """
         Wait until the screen content contains the text string provider.
@@ -439,15 +516,9 @@ class BackendInterface(ABC):
         :return: None
         :rtype: NoneType
         """
-        if self.compare_screen_with_text(text):
-            return
-
-        endtime = time() + timeout
-        while True:
-            self.wait_for_screen_change(endtime - time())
-            if self.compare_screen_with_text(text):
-                return
+        raise NotImplementedError
 
+    @abstractmethod
     def wait_for_text_not_on_screen(self, text: str, timeout: float = 10.0) -> None:
         """
         Wait until the screen content does not contains the text string provider.
@@ -468,14 +539,7 @@ class BackendInterface(ABC):
         :return: None
         :rtype: NoneType
         """
-        if not self.compare_screen_with_text(text):
-            return
-
-        endtime = time() + timeout
-        while True:
-            self.wait_for_screen_change(endtime - time())
-            if not self.compare_screen_with_text(text):
-                return
+        raise NotImplementedError
 
     @abstractmethod
     def get_current_screen_content(self) -> Any:
