@@ -17,13 +17,16 @@ from abc import ABC
 from pathlib import Path
 from tempfile import NamedTemporaryFile
 from time import time
-from typing import Callable, Dict, List, Optional, Union
+from typing import Callable, Dict, Optional, Sequence, Union
+from ledgered.devices import Device
 
 from ragger.backend import BackendInterface, SpeculosBackend
-from ragger.firmware import Firmware
 from ragger.utils import Crop
 
-from .instruction import NavIns, NavInsID
+from .instruction import BaseNavInsID, NavIns, NavInsID
+
+LAST_SCREEN_UPDATE_TIMEOUT = 2
+InstructionType = Union[NavIns, BaseNavInsID]
 
 
 class Navigator(ABC):
@@ -34,33 +37,35 @@ class Navigator(ABC):
 
     def __init__(self,
                  backend: BackendInterface,
-                 firmware: Firmware,
-                 callbacks: Dict[NavInsID, Callable],
+                 device: Device,
+                 callbacks: Dict[BaseNavInsID, Callable],
                  golden_run: bool = False):
         """Initializes the Backend
 
-        :param firmware: Which Backend will be managed
-        :type firmware: Backend
-        :param firmware: Which Firmware will be managed
-        :type firmware: Firmware
+        :param backend: Which Backend will be managed
+        :type backend: BackendInterface
+        :param device: Which Device will be managed
+        :type device: Device
         :param callbacks: Callbacks to use to navigate
-        :type callbacks: Firmware
+        :type callbacks: Device
         :param golden_run: Allows to generate golden snapshots
         :type golden_run: bool
         """
         self._backend = backend
-        self._firmware = firmware
+        self._device = device
         self._callbacks = callbacks
         self._golden_run = golden_run
 
-    def _get_snaps_dir_path(self, path: Path, test_case_name: Path, is_golden: bool) -> Path:
+    def _get_snaps_dir_path(self, path: Path, test_case_name: Union[Path, str],
+                            is_golden: bool) -> Path:
         if is_golden:
             subdir = "snapshots"
         else:
             subdir = "snapshots-tmp"
-        return path / subdir / self._firmware.name / test_case_name
+        return path / subdir / self._device.name / test_case_name
 
-    def _check_snaps_dir_path(self, path: Path, test_case_name: Path, is_golden: bool) -> Path:
+    def _check_snaps_dir_path(self, path: Path, test_case_name: Union[Path, str],
+                              is_golden: bool) -> Path:
         dir_path = self._get_snaps_dir_path(path, test_case_name, is_golden)
         if not dir_path.is_dir():
             if self._golden_run:
@@ -69,7 +74,10 @@ class Navigator(ABC):
                 raise ValueError(f"Golden snapshots directory ({dir_path}) does not exist.")
         return dir_path
 
-    def _init_snaps_temp_dir(self, path: Path, test_case_name: Path, start_idx: int = 0) -> Path:
+    def _init_snaps_temp_dir(self,
+                             path: Path,
+                             test_case_name: Union[Path, str],
+                             start_idx: int = 0) -> Path:
         snaps_tmp_path = self._get_snaps_dir_path(path, test_case_name, False)
         if snaps_tmp_path.exists():
             for file in snaps_tmp_path.iterdir():
@@ -108,14 +116,14 @@ class Navigator(ABC):
 
         assert self._backend.compare_screen_with_snapshot(
             golden, tmp_snap_path=tmp,
-            golden_run=self._golden_run), f"Screen does not match golden {tmp}."
+            golden_run=self._golden_run), f"Screen does not match golden '{tmp}'"
 
-    def add_callback(self, ins_id: NavInsID, callback: Callable, override: bool = True) -> None:
+    def add_callback(self, ins_id: BaseNavInsID, callback: Callable, override: bool = True) -> None:
         """
         Register a new callback.
 
         :param ins_id: The navigation instruction ID which will trigger the callback
-        :type ins_id: NavInsID
+        :type ins_id: BaseNavInsID
         :param callback: The callback to call
         :type callback: Callable
         :param override: Replace an existing callback if the navigation instruction ID already
@@ -134,13 +142,13 @@ class Navigator(ABC):
         self._callbacks[ins_id] = callback
 
     def _run_instruction(self,
-                         instruction: Union[NavIns, NavInsID],
+                         instruction: InstructionType,
                          timeout: float = 10.0,
                          wait_for_screen_change: bool = True,
                          path: Optional[Path] = None,
-                         test_case_name: Optional[Path] = None,
+                         test_case_name: Optional[Union[Path, str]] = None,
                          snap_idx: int = 0) -> None:
-        if isinstance(instruction, NavInsID):
+        if isinstance(instruction, BaseNavInsID):
             instruction = NavIns(instruction)
         if instruction.id not in self._callbacks:
             raise NotImplementedError(f"No callback registered for instruction ID {instruction.id}")
@@ -205,8 +213,8 @@ class Navigator(ABC):
 
     def navigate_and_compare(self,
                              path: Optional[Path],
-                             test_case_name: Optional[Path],
-                             instructions: List[Union[Union[NavIns, NavInsID]]],
+                             test_case_name: Optional[Union[Path, str]],
+                             instructions: Sequence[InstructionType],
                              timeout: float = 10.0,
                              screen_change_before_first_instruction: bool = True,
                              screen_change_after_last_instruction: bool = True,
@@ -218,10 +226,10 @@ class Navigator(ABC):
         :param path: Absolute path to the snapshots directory.
         :type path: Optional[Path]
         :param test_case_name: Relative path to the test case snapshots directory (from path).
-        :type test_case_name: Optional[Path]
+        :type test_case_name: Optional[Union[Path, str]]
         :param instructions: Set of navigation instructions. Navigation instruction IDs are also
                              accepted.
-        :type instructions: List[Union[NavIns, NavInsID]]
+        :type instructions: Sequence[Union[NavIns, BaseNavInsID]]
         :param timeout: Timeout for each navigation step.
         :type timeout: int
         :param screen_change_before_first_instruction: Wait for a screen change before first
@@ -256,8 +264,8 @@ class Navigator(ABC):
                               test_case_name=test_case_name,
                               snap_idx=snap_start_idx)
 
-        for idx, instruction in enumerate(instructions):
-            if idx + 1 != len(instructions) or screen_change_after_last_instruction:
+        for idx, instruction in enumerate(instructions, start=1):
+            if idx != len(instructions) or screen_change_after_last_instruction:
                 # Nominal case, either:
                 # - middle instruction
                 # - last instruction but with screen_change_after_last_instruction=True
@@ -268,7 +276,7 @@ class Navigator(ABC):
                                       wait_for_screen_change=True,
                                       path=path,
                                       test_case_name=test_case_name,
-                                      snap_idx=snap_start_idx + idx + 1)
+                                      snap_idx=snap_start_idx + idx)
             else:
                 # Last instruction case with screen_change_after_last_instruction=False
                 # => no wait_for_screen_change()
@@ -276,12 +284,12 @@ class Navigator(ABC):
                 self._run_instruction(instruction,
                                       timeout,
                                       wait_for_screen_change=False,
-                                      snap_idx=snap_start_idx + idx + 1)
+                                      snap_idx=snap_start_idx + idx)
 
         self._backend.resume_ticker()
 
     def navigate(self,
-                 instructions: List[Union[NavIns, NavInsID]],
+                 instructions: Sequence[InstructionType],
                  timeout: float = 10.0,
                  screen_change_before_first_instruction: bool = True,
                  screen_change_after_last_instruction: bool = True) -> None:
@@ -291,7 +299,7 @@ class Navigator(ABC):
         :param instructions: Set of navigation instructions. Navigation instruction IDs are also
                              accepted, and will be converted into navigation instruction (without
                              any argument)
-        :type instructions: List[Union[NavIns, NavInsID]]
+        :type instructions: Sequence[Union[NavIns, BaseNavInsID]]
         :param timeout: Timeout for each navigation step.
         :type timeout: int
         :param screen_change_before_first_instruction: Wait for a screen change before first
@@ -317,10 +325,10 @@ class Navigator(ABC):
             screen_change_after_last_instruction=screen_change_after_last_instruction)
 
     def navigate_until_snap(self,
-                            navigate_instruction: Union[NavIns, NavInsID],
-                            validation_instruction: Union[NavIns, NavInsID],
+                            navigate_instruction: InstructionType,
+                            validation_instruction: InstructionType,
                             path: Path,
-                            test_case_name: Path,
+                            test_case_name: Union[Path, str],
                             start_img_name: str,
                             last_img_name: str,
                             take_snaps: bool = True,
@@ -339,18 +347,18 @@ class Navigator(ABC):
 
         :param navigate_instruction: Navigation instruction to be performed until last snapshot is found.
                                      Navigation instruction ID is also accepted.
-        :type navigate_instruction: Union[NavIns, NavInsID]
+        :type navigate_instruction: Union[NavIns, BaseNavInsID]
         :param validation_instruction: Navigation instruction to be performed once last snapshot is found.
                                        Navigation instruction ID is also accepted.
-        :type validation_instruction: Union[NavIns, NavInsID]
+        :type validation_instruction: Union[NavIns, BaseNavInsID]
         :param path: Absolute path to the snapshots directory.
         :type path: Path
         :param test_case_name: Relative path to the test case snapshots directory (from path).
-        :type test_case_name: Path
-        :param start_img_name: Index of the first snapshot of the navigation flow.
-        :type start_img_name: int
-        :param last_img_name: Index of the snapshot to look for when navigating.
-        :type last_img_name: int
+        :type test_case_name: Union[Path, str]
+        :param start_img_name: Name of the first snapshot of the navigation flow.
+        :type start_img_name: str
+        :param last_img_name: Name of the snapshot to look for when navigating.
+        :type last_img_name: str
         :param take_snaps: Take temporary snapshots of the screen displayed when navigating.
         :type take_snaps: bool
         :param timeout: Timeout of the navigation loop if last snapshot is not found.
@@ -418,10 +426,9 @@ class Navigator(ABC):
 
             # Make sure there is a screen update after the final action.
             start = time()
-            last_screen_update_timeout = 2
             while self._compare_snap_with_timeout(last_golden_snap, timeout_s=0.5, crop=crop_last):
                 now = time()
-                if (now - start > last_screen_update_timeout):
+                if (now - start > LAST_SCREEN_UPDATE_TIMEOUT):
                     raise TimeoutError(
                         f"Timeout waiting for screen change after last snapshot : {last_golden_snap}"
                     )
@@ -430,14 +437,15 @@ class Navigator(ABC):
         return img_idx
 
     def navigate_until_text_and_compare(self,
-                                        navigate_instruction: Union[NavIns, NavInsID],
-                                        validation_instructions: List[Union[NavIns, NavInsID]],
+                                        navigate_instruction: InstructionType,
+                                        validation_instructions: Sequence[InstructionType],
                                         text: str,
                                         path: Optional[Path] = None,
-                                        test_case_name: Optional[Path] = None,
+                                        test_case_name: Optional[Union[Path, str]] = None,
                                         timeout: int = 300,
                                         screen_change_before_first_instruction: bool = True,
-                                        screen_change_after_last_instruction: bool = True) -> None:
+                                        screen_change_after_last_instruction: bool = True,
+                                        snap_start_idx: int = 0) -> None:
         """
         Navigate until some text is found on the screen content displayed then
         compare each step snapshot with "golden images".
@@ -450,15 +458,15 @@ class Navigator(ABC):
         action is performed on the device.
 
         :param path: Absolute path to the snapshots directory.
-        :type path: Path
+        :type path: Optional[Path]
         :param test_case_name: Relative path to the test case snapshots directory (from path).
-        :type test_case_name: Path
+        :type test_case_name: Optional[Union[Path, str]]
         :param navigate_instruction: Navigation instruction to be performed until the text is found.
                                      Navigation instruction ID is also accepted.
-        :type navigate_instruction: Union[NavIns, NavInsID]
+        :type navigate_instruction: Union[NavIns, BaseNavInsID]
         :param validation_instructions: Navigation instructions to be performed once the text is found.
                                         Navigation instruction IDs are also accepted.
-        :type validation_instructions: List[Union[NavIns, NavInsID]]
+        :type validation_instructions: Sequence[Union[NavIns, BaseNavInsID]]
         :param text: Text string to look for.
         :type text: str
         :param timeout: Timeout for the whole navigation loop.
@@ -467,13 +475,15 @@ class Navigator(ABC):
         :type screen_change_before_first_instruction: bool
         :param screen_change_after_last_instruction: Wait for a screen change after last instruction.
         :type screen_change_after_last_instruction: bool
+        :param snap_start_idx: Index of the first snap for this navigation.
+        :type snap_start_idx: int
 
         :raises TimeoutError: If the text is not found.
 
         :return: None
         :rtype: NoneType
         """
-        idx = 0
+        idx = snap_start_idx
         start = time()
         if not isinstance(self._backend, SpeculosBackend):
             # Update timeout default value for other backends. User needs time to
@@ -531,12 +541,13 @@ class Navigator(ABC):
         self._backend.resume_ticker()
 
     def navigate_until_text(self,
-                            navigate_instruction: Union[NavIns, NavInsID],
-                            validation_instructions: List[Union[NavIns, NavInsID]],
+                            navigate_instruction: InstructionType,
+                            validation_instructions: Sequence[InstructionType],
                             text: str,
                             timeout: int = 300,
                             screen_change_before_first_instruction: bool = True,
-                            screen_change_after_last_instruction: bool = True) -> None:
+                            screen_change_after_last_instruction: bool = True,
+                            snap_start_idx: int = 0) -> None:
         """
         Navigate until some text is found on the screen content displayed.
 
@@ -549,10 +560,10 @@ class Navigator(ABC):
 
         :param navigate_instruction: Navigation instruction to be performed until the text is found.
                                      Navigation instruction ID is also accepted.
-        :type navigate_instruction: Union[NavIns, NavInsID]
+        :type navigate_instruction: Union[NavIns, BaseNavInsID]
         :param validation_instructions: Navigation instructions to be performed once the text is found.
                                         Navigation instruction IDs are also accepted.
-        :type validation_instructions: List[Union[NavIns, NavInsID]]
+        :type validation_instructions: Sequence[Union[NavIns, BaseNavInsID]]
         :param text: Text string to look for.
         :type text: str
         :param timeout: Timeout for the whole navigation loop.
@@ -561,13 +572,20 @@ class Navigator(ABC):
         :type screen_change_before_first_instruction: bool
         :param screen_change_after_last_instruction: Wait for a screen change after last instruction.
         :type screen_change_after_last_instruction: bool
+        :param snap_start_idx: Index of the first snap for this navigation.
+        :type snap_start_idx: int
 
         :raises TimeoutError: If the text is not found.
 
         :return: None
         :rtype: NoneType
         """
-        self.navigate_until_text_and_compare(navigate_instruction, validation_instructions, text,
-                                             None, None, timeout,
+        self.navigate_until_text_and_compare(navigate_instruction,
+                                             validation_instructions,
+                                             text,
+                                             None,
+                                             None,
+                                             timeout,
                                              screen_change_before_first_instruction,
-                                             screen_change_after_last_instruction)
+                                             screen_change_after_last_instruction,
+                                             snap_start_idx=snap_start_idx)
